#!/bin/bash
#
# Startup script for the Altiris Linux Client
#
# chkconfig: 235 88 20
# description: Altiris Network Management Client for Linux
# processname: adlagent
# pidfile: /var/run/adlagent.pid
# config: /opt/altiris/deployment/adlagent/adlagent.conf


# Source function library.
. /etc/rc.d/init.d/functions



# Path to the adlagent binary.
prog=adlagent
instdir=/opt/altiris/deployment/adlagent

RETVAL=0


# Change the major functions into functions.
start() {
	echo -n "Starting $prog: "
	$instdir/$prog -start > /dev/null 2> /dev/null
	RETVAL=$?
	touch /var/lock/subsys/$prog

	if [ $RETVAL = 0 ]; then
	    echo_success
	else
	    echo_failure
	fi;
	return $RETVAL
}
stop() {
	echo -n "Stopping $prog: "
	killproc "$prog"
	RETVAL=$?
	echo
	rm -f "/var/lock/subsys/$prog"
	return $RETVAL
}
status(){
       # get pid of process that created the lock file
       LOCK_PID=`cat /var/run/adlagent.pid`
      
       #see if that process is still running with our name
       ps -p $LOCK_PID | grep adlagent
       if [ "$?" == "0" ]; then
            echo Altiris Deployment agent for Linux is running.
       else
	    echo Altiris Deployment agent for Linux is not running
       fi;
}

# See how we were called.
case "$1" in
  start)
	start
	;;
  stop)
	stop
	;;
  status)
	status 
	;;
  restart)
	stop
	start
	;;
  reload)
	echo -n $"Reloading $prog: "
	killproc $adlagent -HUP
	RETVAL=$?
	echo
	;;
  *)
	echo $"Usage: $prog {start|stop|restart|reload|status}"
	exit 1
esac

exit $RETVAL

